# -*- coding: utf-8 -*-
"""
Views for status API.
"""
import importlib
from django.http import JsonResponse
from django.utils.decorators import classonlymethod
from django.views.generic import View
from django.views.generic.base import ContextMixin

__all__ = ['ProviderAPIView']


class JSONResponseMixin(object):
    """
    A mixin that can be used to render a JSON response.
    """
    def render_to_json_response(self, context, **response_kwargs):
        """
        Returns a JSON response, transforming 'context' to make the payload.
        """
        return JsonResponse(
            self.get_data(context),
            **response_kwargs
        )

    def get_data(self, context):
        """
        Returns an object that will be serialized as JSON by json.dumps().
        """
        return context or {}


class JSONView(JSONResponseMixin, View):
    """
    View that returns a JSON response.
    """
    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, **response_kwargs)


class APIView(JSONView, ContextMixin):
    """
    Base class for API views.
    """
    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def put(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def delete(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)


class ProviderAPIView(APIView):
    """
    Specific class that uses a given provider.
    """
    provider = None
    provider_args = None
    provider_kwargs = None

    def __init__(self, provider, provider_args, provider_kwargs, **kwargs):
        if isinstance(provider, str):
            provider_module, provider_func = provider.rsplit('.', 1)
            module = importlib.import_module(provider_module)
            self.provider = getattr(module, provider_func, None)
        else:
            self.provider = provider

        self.provider_args = provider_args or ()
        self.provider_kwargs = provider_kwargs or {}

        super(ProviderAPIView, self).__init__(**kwargs)

    def get_context_data(self):
        return self.provider(*self.provider_args, **self.provider_kwargs)



# -*- coding: utf-8 -*-
"""Django application config module.
"""

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class Status(AppConfig):
    name = 'status'
    verbose_name = _('Status')



# -*- coding: utf-8 -*-
"""
URLs.
"""
from django.conf.urls import patterns, url

from status.views import ProviderAPIView
from status import settings

urlpatterns = patterns('')

providers = [url(r'^api/{}/?$'.format(a),
                 ProviderAPIView.as_view(provider=p, provider_args=args, provider_kwargs=kwargs),
                 name='api_{}'.format(a))
             for a, p, args, kwargs in settings.CHECK_PROVIDERS]

urlpatterns += providers



# -*- coding: utf-8 -*-
"""
Django Status application.
"""
__version__ = '1.0.2'
__license__ = 'GPLv3'

__author__ = 'José Antonio Perdiguero López'
__email__ = 'perdy.hh@gmail.com'

__url__ = 'https://github.com/PeRDy/django-status'
__description__ = 'Application that provides an API to check the status of some parts and some utilities like ping.'

default_app_config = 'status.apps.Status'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys

from setuptools import setup
from setuptools.command.test import test as TestCommand

import status

with open('requirements.txt', 'r') as f:
    requires = f.read().splitlines()


class Tox(TestCommand):
    user_options = [('tox-args=', 'a', "Arguments to pass to tox")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.tox_args = ''

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import tox
        import shlex
        errno = tox.cmdline(args=shlex.split(self.tox_args))
        sys.exit(errno)


setup(
    name='django-status',
    version=status.__version__,
    description=status.__description__,
    long_description='\n'.join([open('README.rst').read(), open('CHANGELOG').read()]),
    author=status.__author__,
    author_email=status.__email__,
    maintainer=status.__author__,
    maintainer_email=status.__email__,
    url=status.__url__,
    download_url=status.__url__,
    packages=[
        'status',
    ],
    include_package_data=True,
    install_requires=requires,
    license=open('LICENSE').read(),
    zip_safe=False,
    keywords='python, django, database, cache, celery, status, check',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    test_suite='tests',
    tests_require=['tox'],
    cmdclass={'test': Tox},
)




# -*- coding: utf-8 -*-
"""
Built-in check providers.
"""
import datetime
from django.db import connections, OperationalError
from django.core.cache import caches as django_caches, InvalidCacheBackendError
from status.settings import CACHES

from status.utils import FakeChecker

try:
    from celery.task import control

    celery_inspect = control.inspect()
except ImportError:
    celery_inspect = FakeChecker()


def ping(*args, **kwargs):
    """Check if current application is running.

    :return: Pong response.
    """

    return {'pong': True}


def celery(workers, *args, **kwargs):
    """Check if given celery workers are running.

    :param workers: List of workers to be checked.
    :return: Status of each worker.
    """
    try:
        active_workers = celery_inspect.ping()
        workers_status = {w: w in active_workers.keys() for w in workers}
    except AttributeError:
        workers_status = None

    return workers_status


def celery_stats(workers, *args, **kwargs):
    """Retrieve the stats data of given celery workers.

    :param workers: List of workers.
    :return: Stats data of each worker.
    """
    try:
        workers_stats = {k: v for k, v in celery_inspect.stats().items() if k in workers}
    except AttributeError:
        workers_stats = None

    return workers_stats


def databases(*args, **kwargs):
    """Check database status.

    :return: Status of each database.
    """
    status = {}
    for connection in connections.all():
        try:
            connection.connect()
            status[connection.alias] = connection.is_usable()
        except OperationalError:
            status[connection.alias] = False

    return status


def databases_stats(*args, **kwargs):
    """Retrieve the stats data of each database.

    :return: Stats data of each database.
    """
    stats = {}
    for connection in connections.all():
        try:
            connection.connect()
        except OperationalError:
            is_usable = False
        else:
            is_usable = connection.is_usable()
        finally:
            stats[connection.alias] = {
                'vendor': connection.vendor,
                'is_usable': is_usable,
                'allow_thread_sharing': connection.allow_thread_sharing,
                'autocommit': connection.autocommit,
                'commit_on_exit': connection.commit_on_exit,
                'in_atomic_block': connection.in_atomic_block,
                'settings': {k: v for k, v in connection.settings_dict.items() if k not in ('USER', 'PASSWORD')}
            }

    return stats


def caches(*args, **kwargs):
    """Check caches status.

    :return: Status of each cache.
    """
    caches_aliases = CACHES.keys()
    value = datetime.datetime.now().strftime('%d/%m/%Y %H:%M:%S')

    status = {}
    for alias in caches_aliases:
        try:
            cache = django_caches[alias]
            cache.set('django_status_test_cache', value)
            status[alias] = True
        except InvalidCacheBackendError:
            status[alias] = False

    return status



# -*- coding: utf-8 -*-
"""
Settings.
"""

try:
    from django.conf import settings
    INSTALLED_APPS = settings.INSTALLED_APPS
    CACHES = settings.CACHES
except (ImportError, AttributeError):
    settings = None
    INSTALLED_APPS = ()
    CACHES = {}

# Celery application
CELERY_WORKERS = getattr(settings, 'STATUS_CELERY_WORKERS', ())

# Tuple of application name, provider, args, kwargs
CHECK_PROVIDERS = (
    ('ping', 'status.check_providers.ping', None, None),
    ('databases', 'status.check_providers.databases', None, None),
    ('databases/stats', 'status.check_providers.databases_stats', None, None),
    ('caches', 'status.check_providers.caches', None, None),
)
if CELERY_WORKERS:
    CHECK_PROVIDERS += (
        ('celery', 'status.check_providers.celery', None, {'workers': CELERY_WORKERS}),
        ('celery/stats', 'status.check_providers.celery_stats', None, {'workers': CELERY_WORKERS}),
    )
CHECK_PROVIDERS += getattr(settings, 'STATUS_CHECK_PROVIDERS', ())
